# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-09 09:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('speedinfo', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='viewprofiler',
            name='sql_total_count',
            field=models.PositiveIntegerField(default=0, verbose_name=b'SQL total queries count'),
        ),
        migrations.AddField(
            model_name='viewprofiler',
            name='sql_total_time',
            field=models.FloatField(default=0, verbose_name=b'SQL total time'),
        ),
    ]



# coding: utf-8

from django.core.cache import cache
from django.db import IntegrityError


class ProfilerData(object):
    """
    Encapsulates work with the profiling data.
    """
    def add(self, view_name, method, is_anon_call, is_cache_hit, sql_time, sql_count, view_execution_time):
        """Adds profiling data.

        :param str view_name: View name
        :param str method: HTTP method (GET, POST, etc.)
        :param bool is_anon_call: If True, it was an anonymous request
        :param bool is_cache_hit: If True, view response was retrieved from cache
        :param float sql_time: SQL queries execution time
        :param int sql_count: Number of SQL queries
        :param float view_execution_time: View execution time
        """
        from django.db.models import F
        from speedinfo.models import ViewProfiler

        try:
            vp, created = ViewProfiler.objects.get_or_create(view_name=view_name, method=method)
        except IntegrityError:
            # IntegrityError exception raised in case of concurrent access
            # to get_or_create method from another webserver worker process
            vp = ViewProfiler.objects.get(view_name=view_name, method=method)

        ViewProfiler.objects.filter(pk=vp.pk).update(
            anon_calls=F('anon_calls') + (is_anon_call and 1 or 0),
            cache_hits=F('cache_hits') + (is_cache_hit and 1 or 0),
            sql_total_time=F('sql_total_time') + sql_time,
            sql_total_count=F('sql_total_count') + sql_count,
            total_calls=F('total_calls') + 1,
            total_time=F('total_time') + view_execution_time
        )

    def all(self):
        """Returns iterable object over all profiling data.

        :return: profiling data
        :rtype: :class:`typing.Iterable`
        """
        from speedinfo.models import ViewProfiler
        return ViewProfiler.objects.order_by('-total_time')

    def reset(self):
        """Deletes all profiling data"""
        from speedinfo.models import ViewProfiler
        ViewProfiler.objects.all().delete()


class Profiler(object):
    """
    Profiler class to store its state and process the data.
    """
    PROFILER_STATE_CACHE_KEY = 'speedinfo.profiler.is_on'

    def __init__(self):
        self.data = ProfilerData()

    @property
    def is_on(self):
        """Returns state of the profiler.

        :return: state of the profiler
        :rtype: bool
        """
        return cache.get(self.PROFILER_STATE_CACHE_KEY, False)

    @is_on.setter
    def is_on(self, value):
        """Sets state of the profiler.

        :param bool value: State value
        """
        cache.set(self.PROFILER_STATE_CACHE_KEY, value, None)



# coding: utf-8

from django.db import models


class ViewProfiler(models.Model):
    """
    Holds profiler stats grouped by view.
    """
    view_name = models.CharField('View name', max_length=255)
    method = models.CharField('HTTP method', max_length=8)
    anon_calls = models.PositiveIntegerField('Anonymous calls', default=0)
    cache_hits = models.PositiveIntegerField('Cache hits', default=0)
    sql_total_time = models.FloatField('SQL total time', default=0)
    sql_total_count = models.PositiveIntegerField('SQL total queries count', default=0)
    total_calls = models.PositiveIntegerField('Total calls', default=0)
    total_time = models.FloatField('Total time', default=0)

    class Meta:
        verbose_name_plural = 'Views profiler'
        unique_together = ('view_name', 'method')

    @property
    def anon_calls_ratio(self):
        """Anonymous calls ratio.

        :return: anonymous calls ratio percent
        :rtype: float
        """
        if self.total_calls > 0:
            return 100 * self.anon_calls / float(self.total_calls)
        else:
            return 0

    @property
    def cache_hits_ratio(self):
        """Cache hits ratio.

        :return: cache hits ratio percent
        :rtype: float
        """
        if self.total_calls > 0:
            return 100 * self.cache_hits / float(self.total_calls)
        else:
            return 0

    @property
    def sql_time_ratio(self):
        """SQL time per call ratio.

        :return: SQL time per call ratio percent
        :rtype: float
        """
        if self.total_time > 0:
            return 100 * self.sql_total_time / float(self.total_time)
        else:
            return 0

    @property
    def sql_count_per_call(self):
        """SQL queries count per call.

        :return: SQL queries count per call
        :rtype: int
        """
        if self.total_calls > 0:
            return int(round(self.sql_total_count / float(self.total_calls)))
        else:
            return 0

    @property
    def time_per_call(self):
        """Time per call.

        :return: time per call
        :rtype: float
        """
        if self.total_calls > 0:
            return self.total_time / float(self.total_calls)
        else:
            return 0



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-09 17:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('speedinfo', '0003_auto_20171023_1722'),
    ]

    operations = [
        migrations.AlterField(
            model_name='viewprofiler',
            name='anon_calls',
            field=models.PositiveIntegerField(default=0, verbose_name='Anonymous calls'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='cache_hits',
            field=models.PositiveIntegerField(default=0, verbose_name='Cache hits'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='method',
            field=models.CharField(max_length=8, verbose_name='HTTP method'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='sql_total_count',
            field=models.PositiveIntegerField(default=0, verbose_name='SQL total queries count'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='sql_total_time',
            field=models.FloatField(default=0, verbose_name='SQL total time'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='total_calls',
            field=models.PositiveIntegerField(default=0, verbose_name='Total calls'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='total_time',
            field=models.FloatField(default=0, verbose_name='Total time'),
        ),
        migrations.AlterField(
            model_name='viewprofiler',
            name='view_name',
            field=models.CharField(max_length=255, verbose_name='View name'),
        ),
    ]



# coding: utf-8

from collections import namedtuple
from django.conf import settings

SPEEDINFO_CACHED_RESPONSE_ATTR_NAME = getattr(settings, 'SPEEDINFO_CACHED_RESPONSE_ATTR_NAME', 'is_cached')
SPEEDINFO_EXCLUDE_URLS = getattr(settings, 'SPEEDINFO_EXCLUDE_URLS', [])
SPEEDINFO_REPORT_COLUMNS = getattr(settings, 'SPEEDINFO_REPORT_COLUMNS', (
    'view_name', 'method', 'anon_calls_ratio', 'cache_hits_ratio',
    'sql_count_per_call', 'sql_time_ratio', 'total_calls', 'time_per_call', 'total_time'))

ReportColumnFormat = namedtuple('ReportColumnFormat', ['name', 'format', 'attr_name', 'order_field'])
SPEEDINFO_REPORT_COLUMNS_FORMAT = [
    ReportColumnFormat('View name', '{}', 'view_name', 'view_name'),
    ReportColumnFormat('HTTP method', '{}', 'method', 'method'),
    ReportColumnFormat('Anonymous calls', '{:.1f}%', 'anon_calls_ratio', None),
    ReportColumnFormat('Cache hits', '{:.1f}%', 'cache_hits_ratio', None),
    ReportColumnFormat('SQL queries per call', '{}', 'sql_count_per_call', None),
    ReportColumnFormat('SQL time', '{:.1f}%', 'sql_time_ratio', None),
    ReportColumnFormat('Total calls', '{}', 'total_calls', 'total_calls'),
    ReportColumnFormat('Time per call', '{:.8f}', 'time_per_call', 'percall'),
    ReportColumnFormat('Total time', '{:.4f}', 'total_time', 'total_time'),
]
